@@ -17,7 +17,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # 
-# $Id: Document.py,v 1.15 2000/04/06 07:11:05 fredgo Exp $
+# $Id: Document.py,v 1.18.2.5 2000/07/24 09:39:32 fredgo Exp $
 
 ''' This module defines a Document class '''
 
@@ -32,11 +32,11 @@ from Pyblio.GnomeUI.Config import ConfigDialog
 from Pyblio.GnomeUI.Fields import FieldsDialog, EntriesDialog
 
 from Pyblio import Connector, Open, Exceptions, Selection, Sort, Base, Config
-from Pyblio import version, Fields
+from Pyblio import version, Fields, Types
 
 import Pyblio.Style.Utils
 
-import gettext, os, string, copy, types
+import gettext, os, string, copy, types, sys, traceback
 
 _ = gettext.gettext
 
@@ -49,13 +49,13 @@ class Document (Connector.Publisher):
         self.w.connect ('delete_event', self.close_document)
         
         file_menu = [
-            UIINFO_MENU_NEW_ITEM     (_("New"), None, self.new_document),
+            UIINFO_MENU_NEW_ITEM     (_("_New"), None, self.new_document),
             UIINFO_MENU_OPEN_ITEM    (self.ui_open_document),
-            UIINFO_ITEM              (_("Merge with..."),None, self.merge_database),
+            UIINFO_ITEM              (_("_Merge with..."),None, self.merge_database),
             UIINFO_MENU_SAVE_ITEM    (self.save_document),
             UIINFO_MENU_SAVE_AS_ITEM (self.save_document_as),
             UIINFO_SEPARATOR,
-            UIINFO_SUBTREE           (_("Previous Documents"), []),
+            UIINFO_SUBTREE           (_("_Previous Documents"), []),
             UIINFO_SEPARATOR,
             UIINFO_MENU_CLOSE_ITEM   (self.close_document),
             UIINFO_MENU_EXIT_ITEM    (self.exit_application)
@@ -75,32 +75,32 @@ class Document (Connector.Publisher):
             UIINFO_MENU_CLEAR_ITEM      (self.clear_entries),
             UIINFO_MENU_SELECT_ALL_ITEM (self.select_all_entries),
             UIINFO_SEPARATOR,
-            UIINFO_ITEM_STOCK(_("Add..."), None, self.add_entry,    STOCK_MENU_NEW),
-            UIINFO_ITEM      (_("Edit..."),None, self.edit_entry),
-            UIINFO_ITEM_STOCK(_("Delete"), None, self.delete_entry, STOCK_MENU_TRASH),
+            UIINFO_ITEM_STOCK(_("_Add..."), None, self.add_entry,    STOCK_MENU_NEW),
+            UIINFO_ITEM      (_("_Edit..."),None, self.edit_entry),
+            UIINFO_ITEM_STOCK(_("_Delete"), None, self.delete_entry, STOCK_MENU_TRASH),
             UIINFO_SEPARATOR,
             UIINFO_MENU_FIND_ITEM (self.find_entries),
-            UIINFO_ITEM           (_("Sort..."),None, self.sort_entries),
+            UIINFO_ITEM           (_("S_ort..."),None, self.sort_entries),
             ]
 
         cite_menu = [
-            UIINFO_ITEM_STOCK(_("Cite"), None, self.lyx_cite,            STOCK_MENU_CONVERT),
-            UIINFO_ITEM_STOCK(_("Format..."), None, self.format_entries, STOCK_MENU_REFRESH),
+            UIINFO_ITEM_STOCK(_("_Cite"), None, self.lyx_cite,            STOCK_MENU_CONVERT),
+            UIINFO_ITEM_STOCK(_("_Format..."), None, self.format_entries, STOCK_MENU_REFRESH),
             ]
 
         settings_menu = [
-            UIINFO_ITEM      (_("Fields..."),  None, self.set_fields),
-            UIINFO_ITEM      (_("Entries..."), None, self.set_entries),
+            UIINFO_ITEM      (_("_Fields..."),  None, self.set_fields),
+            UIINFO_ITEM      (_("_Entries..."), None, self.set_entries),
             UIINFO_SEPARATOR,
             UIINFO_MENU_PREFERENCES_ITEM   (self.set_preferences),
             ]
             
         menus = [
-            UIINFO_SUBTREE (_("File"),     file_menu),
-            UIINFO_SUBTREE (_("Edit"),     edit_menu),
-            UIINFO_SUBTREE (_("Cite"),     cite_menu),
-            UIINFO_SUBTREE (_("Settings"), settings_menu),
-            UIINFO_SUBTREE (_("Help"),     help_menu)
+            UIINFO_SUBTREE (_("_File"),     file_menu),
+            UIINFO_SUBTREE (_("_Edit"),     edit_menu),
+            UIINFO_SUBTREE (_("_Cite"),     cite_menu),
+            UIINFO_SUBTREE (_("_Settings"), settings_menu),
+            UIINFO_SUBTREE (_("_Help"),     help_menu)
             ]
         
         toolbar = [
@@ -166,7 +166,8 @@ class Document (Connector.Publisher):
         self.sort_dg   = None
         self.lyx       = None
         self.changed   = 0
-
+        self.directory = None
+        
         self.redisplay_index ()
         return
 
@@ -194,12 +195,14 @@ class Document (Connector.Publisher):
 
         menuinfo = []
         for item in history:
-            def callback (widget, self = self, file = item):
+            def callback (widget, self = self, file = item [0], how = item [1]):
                 if not self.confirm (): return
-                self.open_document (file)
+                self.open_document (file, how)
                 return
 
-            menuinfo.append (UIINFO_ITEM_STOCK (item, None, callback, STOCK_MENU_OPEN))
+            filename = item [0]
+            
+            menuinfo.append (UIINFO_ITEM_STOCK (filename, None, callback, STOCK_MENU_OPEN))
 
         self.w.insert_menus (_("File") + '/' + _("Previous Documents") + '/',
                              menuinfo)
@@ -212,8 +215,9 @@ class Document (Connector.Publisher):
         
         if changed != -1:
             self.changed = changed
-        
+
         self.index.display (self.selection.iterator (self.data.iterator ()))
+        
         self.update_status ()
         return
 
@@ -232,8 +236,12 @@ class Document (Connector.Publisher):
             entries = self.data.keys ()
 
         url = Fields.URL (style)
-        
-        Pyblio.Style.Utils.generate (url, format, self.data, entries, file)
+
+        try:
+            Pyblio.Style.Utils.generate (url, format, self.data, entries, file)
+        except RuntimeError, err:
+            print err
+            self.w.error (_("Error while parsing `%s':\n%s") % (style, err))
         return
 
 
@@ -344,11 +352,16 @@ class Document (Connector.Publisher):
         if not self.confirm (): return
 
         # get a new file name
-        (url, how) = FileSelector.URLFileSelection (_("Open file")).run ()
+        (url, how) = FileSelector.URLFileSelection (_("Open file"),
+                                                    directory = self.directory).run ()
 
         if url is None: return
-
         self.open_document (url, how)
+
+        # memorize the current path in the case of a file
+        url = Fields.URL (url)
+        if url.url [0] == 'file':
+            self.directory = os.path.split (url.url [2]) [0] + '/'
         return
 
     
@@ -385,13 +398,16 @@ class Document (Connector.Publisher):
         Utils.set_cursor (self.w, 'clock')
         try:
             try:
-                self.data.update ()
+                self.data.update (self.selection.sort)
             except (OSError, IOError), error:
                 Utils.set_cursor (self.w, 'normal')
                 self.w.error (_("Unable to save `%s':\n%s") % (str (self.data.key),
                                                                str (error)))
                 return
         except:
+            etype, value, tb = sys.exc_info ()
+            traceback.print_exception (etype, value, tb)
+            
             Utils.set_cursor (self.w, 'normal')
             self.w.error (_("An internal error occured during saving\nTry to Save As..."))
             return
@@ -404,7 +420,7 @@ class Document (Connector.Publisher):
     
     def save_document_as (self, * arg):
         # get a new file name
-        (url, how) = FileSelector.URLFileSelection (_("Open file"),
+        (url, how) = FileSelector.URLFileSelection (_("Save As..."),
                                                     url = FALSE, has_auto = FALSE).run ()
 
         if url is None: return
@@ -421,15 +437,27 @@ class Document (Connector.Publisher):
             return
 
         Utils.set_cursor (self.w, 'clock')
-        
-        Open.bibwrite (self.data.iterator (), out = file, how = how)
+
+        iterator = Selection.Selection (sort = self.selection.sort)
+        Open.bibwrite (iterator.iterator (self.data.iterator ()),
+                       out = file, how = how)
         file.close ()
         
         if self.data.key is None:
             # we wrote an anonymous database. Lets reopen it !
-            self.data = Open.bibopen (url, how = how)
-            self.redisplay_index ()
+            try:
+                self.data = Open.bibopen (url, how = how)
+                
+            except (Exceptions.ParserError,
+                    Exceptions.FormatError,
+                    Exceptions.FileError), error:
+                    
+                Utils.set_cursor (self.w, 'normal')
+                Utils.error_dialog (_("Reopen error"), error,
+                                    parent = self.w)
+                return
             
+            self.redisplay_index ()
             self.issue ('open-document', self)
             
         Utils.set_cursor (self.w, 'normal')
@@ -440,7 +468,7 @@ class Document (Connector.Publisher):
                                       
     def close_document (self, * arg):
         self.issue ('close-document', self)
-        return
+        return 1
 
 
     def close_document_request (self):
@@ -474,11 +502,13 @@ class Document (Connector.Publisher):
             self.data.add (entry)
 
         self.redisplay_index ()
+        self.index.set_scroll (entries [-1])
         return
 
                 
     def cut_entry (self, * arg):
         entries = self.index.selection ()
+        if not entries: return
         
         self.index.selection_copy (entries)
         for entry in entries:
@@ -521,7 +551,7 @@ class Document (Connector.Publisher):
     def add_entry (self, * arg):
         entry = self.data.new_entry (Config.get ('base/defaulttype').data)
         
-        edit = Editor.Editor (self.data, entry, self.w)
+        edit = Editor.Editor (self.data, entry, self.w, _("Create new entry"))
         edit.Subscribe ('commit-edition', self.commit_edition)
         return
 
@@ -557,18 +587,21 @@ class Document (Connector.Publisher):
         else:
             self.data.add (new)
 
-        self.redisplay_index (1)
         self.freeze_display (None)
+
+        self.redisplay_index (1)
+        self.index.select_item (new)
         return
     
     
     def delete_entry (self, * arg):
         ''' removes the selected list of items after confirmation '''
         entries = self.index.selection ()
-
         l = len (entries)
         if l == 0: return
-        
+
+        offset = self.index.get_item_position (entries [-1])
+
         if l > 1:
             question = _("Remove all the %d entries ?") % len (entries)
         else:
@@ -582,6 +615,7 @@ class Document (Connector.Publisher):
             del self.data [entry.key]
             
         self.redisplay_index (1)
+        self.index.select_item (offset)
         return
     
     
@@ -663,6 +697,10 @@ class Document (Connector.Publisher):
         # 2.- Proportion betzeen list and text
         height = self.paned.children () [0].get_allocation () [3]
         config.set_int ('Pybliographic/UI/Paned', height)
+
+        # updates the index's config
+        self.index.update_configuration ()
+        
         config.sync ()
         return
 
